
import java.util.Arrays;

public class Brano {
    public final String titolo;
    public final String genere;
    // Durata in secondi
    public final int durata;
    public final Artista[] artisti;

    public Brano(String titolo, String genere, int minuti, int secondi, Artista... artisti) {
        this(titolo, genere, minuti * 60 + secondi, artisti);
    }

    public Brano(String titolo, String genere, int durata, Artista... artisti) {
        if(artisti.length > 5) {
            throw new IllegalArgumentException("Un brano non può avere più di 5 artisti");
        }
        if(durata < 0) {
            throw new IllegalArgumentException("Un brano non può avere durata negativa");
        }
        this.titolo = titolo;
        this.genere = genere;
        this.durata = durata;
        this.artisti = artisti;
    }

    @Override
    public String toString() {
        return String.format(
                "(%s, %d:%02d) \"%s\" - %s",
                genere,
                durata / 60,
                durata % 60,
                titolo,
                String.join(", ", Arrays.stream(artisti).map(artista -> artista.toString()).toList())
        );
    }
    
}
