import java.util.Random;

public class Playlist {
    private Nodo<Brano> corrente = null;
    private Nodo<Brano> testa = null;
    private Nodo<Brano> coda = null;
    private boolean shuffle = false;
    private Random random = null;
    private Nodo<Nodo<Brano>> pilaPrecedenti = null;
    private Nodo<Nodo<Brano>> pilaProssimi = null;
    private int dimensione = 0;

    public Playlist(Brano... brani) {
        for(Brano brano : brani) {
            if(brano == null) continue;
            aggiungiInCoda(brano);
        }
    }

    public void aggiungiInCoda(Brano brano) {
        dimensione++;
        if(corrente != null) {
            Nodo<Brano> cursore = corrente;
            while(cursore.prossimo != null) cursore = cursore.prossimo;
            cursore.prossimo = new Nodo<Brano>(brano);
            cursore.prossimo.precedente = cursore;
            coda = cursore.prossimo;
        } else {
            corrente = new Nodo<Brano>(brano);
            testa = corrente;
            coda = corrente;
        }
    }

    private Nodo<Brano> branoCasuale() {
        Nodo<Brano> cursore = testa;
        if(random == null) random = new Random();
        int pos = random.nextInt(0, dimensione);
        for(int i = 0; i < pos; i++) {
            cursore = cursore.prossimo;
        }
        return cursore;
    }

    public Brano precedente() {
        if(dimensione == 0) return null;


        if(pilaPrecedenti != null) {
            aggiungiInProssimi(corrente);
            corrente = pilaPrecedenti.valore;
            pilaPrecedenti = pilaPrecedenti.prossimo;
            if(pilaPrecedenti != null) {
                pilaPrecedenti.precedente = null;
            }
            return corrente.valore;
        }

        if(shuffle) {
            aggiungiInProssimi(corrente);
            corrente = branoCasuale();
            return corrente.valore;
        }

        if(corrente == null || corrente.precedente == null) return null;
        aggiungiInProssimi(corrente);
        corrente = corrente.precedente;
        return corrente.valore;
    }

    private void aggiungiInProssimi(Nodo<Brano> nodo) {
        Nodo<Nodo<Brano>> nuovoProssimo = new Nodo<Nodo<Brano>>(nodo);
        if(pilaProssimi != null) {
            nuovoProssimo.prossimo = pilaProssimi;
            nuovoProssimo.prossimo.precedente = nuovoProssimo;
        }
        pilaProssimi = nuovoProssimo;
    }

    public Brano corrente() {
        if(corrente == null) return null;
        return corrente.valore;
    }
 
    public Brano prossimo() {
        if(dimensione == 0) return null;

        if(pilaProssimi != null) {
            aggiungiInPrecedenti(corrente);
            corrente = pilaProssimi.valore;
            pilaProssimi = pilaProssimi.prossimo;
            if(pilaProssimi != null) {
                pilaProssimi.precedente = null;
            }
            return corrente.valore;
        }

        if(shuffle) {
            aggiungiInPrecedenti(corrente);
            corrente = branoCasuale();
            return corrente.valore;
        }

        if(corrente == null || corrente.prossimo == null) return null;
        aggiungiInPrecedenti(corrente);
        corrente = corrente.prossimo;
        return corrente.valore;
    }

    private void aggiungiInPrecedenti(Nodo<Brano> nodo) {
        Nodo<Nodo<Brano>> nuovoPrecedente = new Nodo<Nodo<Brano>>(corrente);
        if(pilaPrecedenti != null) {
            nuovoPrecedente.prossimo = pilaPrecedenti;
            nuovoPrecedente.prossimo.precedente = nuovoPrecedente;
        }
        pilaPrecedenti = nuovoPrecedente;
    }

    public void repeat(boolean abilitato) {
        // In un contesto reale non modificherei la lista in se ma cambierei la logica solo
        // quando devo selezionare la traccia successiva o precedente. Per scopi didattici,
        // rendo la lista una lista circolare.
        if(abilitato) {
            testa.precedente = coda;
            coda.prossimo = testa;
        } else {
            testa.precedente = null;
            coda.prossimo = null;
        }
        System.out.println("Repeat: " + abilitato);
    }

    public void shuffle(boolean abilitato) {
        shuffle = abilitato;
        System.out.println("Shuffle: " + abilitato);
    }

    private class Nodo<T> {
        public T valore;
        public Nodo<T> prossimo;
        public Nodo<T> precedente;

        public Nodo(T valore) {
            this.valore = valore;
        }
    }
}
